OEP: 0
Title: OEP Purpose and Guidelines
Author: Open World Agents Team
Status: Active
Type: Process
Created: 07-Jun-2025
Post-History: 07-Jun-2025

Abstract
========

OEP stands for Open World Agents Enhancement Proposal. An OEP is a design document providing information to the Open World Agents community, or describing a new feature for OWA or its processes or environment. The OEP should provide a concise technical specification of the feature and a rationale for the feature.

We intend OEPs to be the primary mechanisms for proposing major new features, for collecting community input on an issue, and for documenting the design decisions that have gone into OWA. The OEP author is responsible for building consensus within the community and documenting dissenting opinions.

Because the OEPs are maintained as text files in a versioned repository, their revision history is the historical record of the feature proposal.

What is an OEP?
===============

OEP stands for Open World Agents Enhancement Proposal. An OEP is a design document providing information to the Open World Agents community, or describing a new feature for the framework or its processes or environment. The OEP should provide a concise technical specification of the feature and a rationale for the feature.

We intend OEPs to be the primary mechanisms for proposing major new features, for collecting community input on an issue, and for documenting the design decisions that have gone into Open World Agents. The OEP author is responsible for building consensus within the community and documenting dissenting opinions.

Because the OEPs are maintained as text files in a versioned repository, their revision history is the historical record of the feature proposal. This historical record is available by the normal git commands for retrieving older revisions, and can also be browsed on GitHub.

OEP Audience
============

The typical primary audience for OEPs are the core developers of Open World Agents and the broader community of contributors and users.

The Open World Agents community may use the process to document expected API conventions and to manage complex design coordination problems that require collaboration across multiple projects and environment plugins.

OEP Types
=========

There are three kinds of OEP:

1. A **Standards Track** OEP describes a new feature or implementation for Open World Agents. It may also describe an interoperability standard that will be supported in the framework.

2. An **Informational** OEP describes an Open World Agents design issue, or provides general guidelines or information to the community, but does not propose a new feature. Informational OEPs do not necessarily represent a community consensus or recommendation, so users and implementers are free to ignore Informational OEPs or follow their advice.

3. A **Process** OEP describes a process surrounding Open World Agents, or proposes a change to (or an event in) a process. Process OEPs are like Standards Track OEPs but apply to areas other than the framework itself. They may propose an implementation, but not to OWA's codebase; they often require community consensus; unlike Informational OEPs, they are more than recommendations, and users are typically not free to ignore them. Examples include procedures, guidelines, changes to the decision-making process, and changes to the tools or environment used in OWA development.

OEP Workflow
============

The OEP process begins with a new idea for Open World Agents. It is highly recommended that a single OEP contain a single key proposal or new idea; the more focused the OEP, the more successful it tends to be. Most enhancements and bug fixes don't need an OEP and can be submitted directly to the GitHub issue tracker.

Each OEP must have a champion – someone who writes the OEP using the style and format described below, shepherds the discussions in the appropriate forums, and attempts to build community consensus around the idea. The OEP champion (a.k.a. Author) should first attempt to ascertain whether the idea is OEP-able.

Vetting an idea publicly before going as far as writing an OEP is meant to save the potential author time. Many ideas have been brought forward for changing Open World Agents that have been rejected for various reasons. Asking the community first if an idea is original helps prevent too much time being spent on something that is guaranteed to be rejected based on prior discussions.

Once the champion has asked the community whether an idea has any chance of acceptance, a draft OEP should be presented. This gives the author a chance to flesh out the draft OEP to make it properly formatted, of high quality, and to address initial concerns about the proposal.

Submitting an OEP
=================

Following the above initial discussion, the workflow is as follows:

The standard OEP workflow is:

* You, the OEP author, fork the Open World Agents repository, and create a file named `oep-NNNN.md` that contains your new OEP. Use the next available OEP number not used by a published or in-PR OEP.
* In the "OEP:" header field, enter the OEP number that matches your filename as your draft OEP number.
* In the "Type:" header field, enter "Standards Track", "Informational", or "Process" as appropriate, and for the "Status:" field enter "Draft".
* Push this to your GitHub fork and submit a pull request.
* The OEP editors review your PR for structure, formatting, and other errors. Approval criteria are:
  * It is sound and complete. The ideas must make technical sense.
  * The title accurately describes the content.
  * The OEP's language (spelling, grammar, sentence structure, etc.) should be correct and conformant.

Once the review process is complete, and the OEP editors approve it, they will assign your OEP a number and merge your pull request.

The OEP editors will not unreasonably deny publication of an OEP. Reasons for denying OEP status include duplication of effort, being technically unsound, not providing proper motivation or addressing backwards compatibility, or not in keeping with the Open World Agents philosophy.

As updates are necessary, the OEP author can check in new versions if they have write access to the repository.

Standards Track OEPs consist of two parts, a design document and a reference implementation. It is generally recommended that at least a prototype implementation be co-developed with the OEP, as ideas that sound good in principle sometimes turn out to be impractical when subjected to the test of implementation.

OEP Review & Resolution
=======================

Once the authors have completed an OEP, they may request a review for style and consistency from the OEP editors. However, content review and acceptance of the OEP is ultimately the responsibility of the core maintainers.

For an OEP to be accepted it must meet certain minimum criteria. It must be a clear and complete description of the proposed enhancement. The enhancement must represent a net improvement. The proposed implementation, if applicable, must be solid and must not complicate the framework unduly. Finally, a proposed enhancement must be consistent with the Open World Agents philosophy and design principles.

Once an OEP has been accepted, the reference implementation must be completed. When the reference implementation is complete and incorporated into the main source code repository, the status will be changed to "Final".

An OEP can also be assigned the status "Deferred". The OEP author or an editor can assign the OEP this status when no progress is being made on the OEP.

An OEP can also be "Rejected". Perhaps after all is said and done it was not a good idea. It is still important to have a record of this fact. The "Withdrawn" status is similar - it means that the OEP author themselves has decided that the OEP is actually a bad idea, or has accepted that a competing proposal is a better alternative.

When an OEP is Accepted, Rejected or Withdrawn, the OEP should be updated accordingly.

Some Informational and Process OEPs may also have a status of "Active" if they are never meant to be completed, like this OEP.

What belongs in a successful OEP?
=================================

Each OEP should have the following parts/sections:

1. **Preamble** – Headers containing meta-data about the OEP, including the OEP number, a short descriptive title, the names and contact info for each author, etc.

2. **Abstract** – a short (~200 word) description of the technical issue being addressed.

3. **Motivation** – The motivation is critical for OEPs that want to change the Open World Agents framework. It should clearly explain why the existing specification is inadequate to address the problem that the OEP solves.

4. **Rationale** – The rationale fleshes out the specification by describing why particular design decisions were made. It should describe alternate designs that were considered and related work.

5. **Specification** – The technical specification should describe the syntax and semantics of any new feature. The specification should be detailed enough to allow competing, interoperable implementations.

6. **Backwards Compatibility** – All OEPs that introduce backwards incompatibilities must include a section describing these incompatibilities and their severity. The OEP must explain how the author proposes to deal with these incompatibilities.

7. **Security Implications** – If there are security concerns in relation to the OEP, those concerns should be explicitly written out to make sure reviewers are aware of them.

8. **How to Teach This** – For an OEP that adds new functionality or changes behavior, it is helpful to include a section on how to teach users, new and experienced, how to apply the OEP to their work.

9. **Reference Implementation** – The reference implementation must be completed before any OEP is given status "Final", but it need not be completed before the OEP is accepted.

10. **Rejected Ideas** – Throughout the discussion of an OEP, various ideas will be proposed which are not accepted. Those rejected ideas should be recorded along with the reasoning as to why they were rejected.

11. **Open Issues** – While an OEP is in draft, ideas can come up which warrant further discussion. Those ideas should be recorded so people know that they are being thought about.

12. **Copyright/license** – Each new OEP must be placed under an appropriate open source license.

OEP Header Preamble
===================

Each OEP must begin with a header preamble. The headers must appear in the following order. Headers marked with "*" are optional and are described below. All other headers are required.

```
OEP: <oep number>
Title: <oep title>
Author: <list of authors' names and optionally, email addrs>
Status: <Draft | Active | Accepted | Deferred | Rejected | Withdrawn | Final | Superseded>
Type: <Standards Track | Informational | Process>
* Requires: <oep numbers>
Created: <date created on, in dd-mmm-yyyy format>
* Post-History: <dates, in dd-mmm-yyyy format>
* Replaces: <oep number>
* Superseded-By: <oep number>
```

The Author header lists the names, and optionally the email addresses of all the authors/owners of the OEP.

The Type header specifies the type of OEP: Standards Track, Informational, or Process.

The Created header records the date that the OEP was assigned a number, while Post-History is used to record when new versions of the OEP are posted.

Standards Track OEPs will typically indicate which version of Open World Agents the feature will be released with.

OEPs may have a Requires header, indicating the OEP numbers that this OEP depends on.

OEPs may also have a Superseded-By header indicating that an OEP has been rendered obsolete by a later document; the value is the number of the OEP that replaces the current document. The newer OEP must have a Replaces header containing the number of the OEP that it rendered obsolete.

Open World Agents Philosophy
============================

Open World Agents is built on several core principles that guide development decisions:

**Real-time Performance**: The framework prioritizes low-latency, real-time interactions suitable for desktop automation and agent control. Target latency is under 30ms for critical operations.

**Asynchronous Design**: Unlike traditional step-based environments, OWA uses an event-driven, asynchronous architecture that mirrors real-world interactions through Callables, Listeners, and Runnables.

**Modular Plugin System**: The framework supports dynamic plugin activation and registration, allowing users to extend functionality without modifying core components.

**Multimodal Data Handling**: OWA provides comprehensive support for capturing, storing, and processing multimodal desktop data including screen capture, keyboard/mouse events, audio, and window information.

**Community-Driven**: The framework is designed to be extensible by the community, with clear interfaces for creating custom environment plugins.

**High Performance**: Emphasis on performance optimization, particularly for screen capture and data processing, achieving significant speed improvements over alternatives.

**Open Standards**: Use of open formats like MCAP for data storage and clear, documented APIs for interoperability.

OEP Template
============

The following template should be used for new OEPs:

```markdown
OEP: <REQUIRED: oep number>
Title: <REQUIRED: oep title>
Author: <REQUIRED: list of authors' names and optionally, email addrs>
Status: <REQUIRED: Draft | Active | Accepted | Deferred | Rejected | Withdrawn | Final | Superseded>
Type: <REQUIRED: Standards Track | Informational | Process>
* Requires: <oep numbers>
Created: <date created on, in dd-mmm-yyyy format>
* Post-History: <dates, in dd-mmm-yyyy format>
* Replaces: <oep number>
* Superseded-By: <oep number>

Abstract
========

[A short (~200 word) description of the technical issue being addressed.]

Motivation
==========

[Clearly explain why the existing framework specification is inadequate to address the problem that the OEP solves.]

Rationale
=========

[Describe why particular design decisions were made.]

Specification
=============

[Describe the syntax and semantics of any new feature.]

Backwards Compatibility
=======================

[Describe potential impact and severity on pre-existing code.]

Security Implications
=====================

[How could a malicious user take advantage of this new feature?]

How to Teach This
=================

[How to teach users, new and experienced, how to apply the OEP to their work.]

Reference Implementation
========================

[Link to any existing implementation and details about its state, e.g. proof-of-concept.]

Rejected Ideas
==============

[Why certain ideas that were brought while discussing this OEP were not ultimately pursued.]

Open Issues
===========

[Any points that are still being decided/discussed.]

References
==========

[A collection of references cited in the OEP.]

Copyright
=========

This document is placed in the public domain or under the
CC0-1.0-Universal license, whichever is more permissive.
```

References
==========

This OEP format is inspired by Python Enhancement Proposals (PEPs) and ROS Enhancement Proposals (REPs), adapted for the Open World Agents community.

- Python Enhancement Proposals: https://github.com/python/peps
- ROS Enhancement Proposals: https://github.com/ros-infrastructure/rep

Copyright
=========

This document is placed in the public domain or under the CC0-1.0-Universal license, whichever is more permissive.
